from mayan.apps.dynamic_search.tests.mixins import SearchTestMixin

from ..permissions import permission_document_view
from ..search import document_search

from .base import GenericDocumentViewTestCase


class DocumentSearchTestCase(SearchTestMixin, GenericDocumentViewTestCase):
    auto_upload_test_document = False

    def _do_test_search(self, query):
        terms = str(tuple((query.values()))[0]).strip()
        self.assertTrue(terms is not None)
        self.assertTrue(terms != '')

        return self.search_backend.search(
            search_model=document_search, query=query,
            user=self._test_case_user
        )

    def setUp(self):
        super().setUp()
        self._upload_test_document()

    def test_document_search_by_datetime_created_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={
                'datetime_created': self.test_document.datetime_created.isoformat()
            }
        )
        self.assertFalse(self.test_document in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_search_by_datetime_created_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'datetime_created': self.test_document.datetime_created.isoformat()
            }
        )
        self.assertTrue(self.test_document in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_search_by_datetime_created_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        self.test_document.delete()

        self._clear_events()

        queryset = self._do_test_search(
            query={
                'datetime_created': self.test_document.datetime_created.isoformat()
            }
        )
        self.assertTrue(self.test_document not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_search_by_document_file_checksum_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={'files__checksum': self.test_document_file.checksum}
        )
        self.assertFalse(self.test_document in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_search_by_document_file_checksum_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={'files__checksum': self.test_document_file.checksum}
        )
        self.assertTrue(self.test_document in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_search_by_document_file_checksum_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        self.test_document.delete()

        self._clear_events()

        queryset = self._do_test_search(
            query={'files__checksum': self.test_document_file.checksum}
        )
        self.assertTrue(self.test_document not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_search_by_document_file_filename_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={'files__filename': self.test_document_file.filename}
        )
        self.assertFalse(self.test_document in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_search_by_document_file_filename_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={'files__filename': self.test_document_file.filename}
        )
        self.assertTrue(self.test_document in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_search_by_document_file_filename_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        self.test_document.delete()

        self._clear_events()

        queryset = self._do_test_search(
            query={'files__filename': self.test_document_file.filename}
        )
        self.assertTrue(self.test_document not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_search_by_document_file_mime_type_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={'files__mimetype': self.test_document_file.mimetype}
        )
        self.assertFalse(self.test_document in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_search_by_document_file_mime_type_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={'files__mimetype': self.test_document_file.mimetype}
        )
        self.assertTrue(self.test_document in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_search_by_document_file_mime_type_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        self.test_document.delete()

        self._clear_events()

        queryset = self._do_test_search(
            query={'files__mimetype': self.test_document_file.mimetype}
        )
        self.assertTrue(self.test_document not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_search_by_description_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={'description': self.test_document.description}
        )
        self.assertFalse(self.test_document in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_search_by_description_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={'description': self.test_document.description}
        )
        self.assertTrue(self.test_document in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_search_by_description_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        self.test_document.delete()

        self._clear_events()

        queryset = self._do_test_search(
            query={'description': self.test_document.description}
        )
        self.assertTrue(self.test_document not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_search_by_label_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={'label': self.test_document.label}
        )
        self.assertFalse(self.test_document in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_search_by_label_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={'label': self.test_document.label}
        )
        self.assertTrue(self.test_document in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_search_by_label_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        self.test_document.delete()

        self._clear_events()

        queryset = self._do_test_search(
            query={'label': self.test_document.label}
        )
        self.assertTrue(self.test_document not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_search_by_uuid_no_permission(self):
        self._clear_events()

        queryset = self._do_test_search(
            query={'uuid': self.test_document.uuid}
        )
        self.assertFalse(self.test_document in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_search_by_uuid_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        self._clear_events()

        queryset = self._do_test_search(
            query={'uuid': self.test_document.uuid}
        )
        self.assertTrue(self.test_document in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_search_by_uuid_with_access(self):
        self.grant_access(
            obj=self.test_document, permission=permission_document_view
        )

        self.test_document.delete()

        self._clear_events()

        queryset = self._do_test_search(
            query={'uuid': self.test_document.uuid}
        )
        self.assertTrue(self.test_document not in queryset)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from django.db.models import Max


class HooksModelMixin:
    @classmethod
    def _execute_hooks(cls, hook_list, **kwargs):
        result = None

        for hook in hook_list:
            result = hook(**kwargs)
            if result:
                kwargs.update(result)

        return result

    @classmethod
    def _insert_hook_entry(cls, hook_list, func, order=None):
        order = order or len(hook_list)
        hook_list.insert(order, func)


class PagedModelMixin:
    def get_pages_last_number(self):
        last_page_number = self.siblings.aggregate(
            page_number_maximum=Max('page_number')
        )['page_number_maximum']

        return last_page_number

    @property
    def siblings(self):
        filter_kwargs = {
            self._paged_model_parent_field: getattr(self, self._paged_model_parent_field)
        }

        return self._meta.default_manager.filter(**filter_kwargs)



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0036_auto_20161222_0534'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document',
            name='language',
            field=models.CharField(
                blank=True, default='eng', max_length=8,
                verbose_name='Language'
            ),
        ),
    ]



from mayan.apps.testing.tests.base import GenericViewTestCase

from ..dashboards import dashboard_administrator

from .mixins import DashboardViewTestMixin


class DashboardViewTestCase(DashboardViewTestMixin, GenericViewTestCase):
    test_dashboard = dashboard_administrator

    def test_dashboard_detail_view(self):
        self._clear_events()

        response = self._request_test_dashboard_detail_view()
        self.assertEqual(response.status_code, 200)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_dashboard_list_view(self):
        self._clear_events()

        response = self._request_test_dashboard_list_view()
        self.assertContains(
            response=response, status_code=200,
            text=self.test_dashboard.label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



import pyotp

from django.conf import settings
from django.db import models
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from .events import event_otp_disabled, event_otp_enabled


class UserOTPData(models.Model):
    """
    This model stores OTP configurations for a user account.
    """
    user = models.OneToOneField(
        on_delete=models.CASCADE, related_name='otp_data',
        to=settings.AUTH_USER_MODEL, unique=True, verbose_name=_('User')
    )
    secret = models.CharField(
        blank=True, max_length=96, verbose_name=_(
            '16 character, base32 encoded random string.'
        )
    )

    class Meta:
        verbose_name = _('User OTP data')
        verbose_name_plural = _('Users OTP data')

    def disable(self):
        self.secret = ''
        self.save()
        event_otp_disabled.commit(
            actor=self.user, target=self.user
        )

    def enable(self, secret, token):
        if self.verify_token(secret=secret, token=token):
            self.secret = secret
            self.save()

            event_otp_enabled.commit(
                actor=self.user, target=self.user
            )

    def get_absolute_url(self):
        return reverse(viewname='authentication_otp:otp_detail')

    def is_enabled(self):
        if self.secret:
            return True
        else:
            return False

    def natural_key(self):
        return self.user.natural_key()
    natural_key.dependencies = [settings.AUTH_USER_MODEL]

    def verify_token(self, token, secret=None):
        if not secret:
            secret = self.secret

        totp = pyotp.TOTP(s=secret)
        return token == totp.now()



import logging

from django.contrib.auth import get_user_model
from django.core import management
from django.db import models

from .settings import setting_email, setting_password, setting_username

logger = logging.getLogger(name=__name__)


class AutoAdminSingletonManager(models.Manager):
    def create_autoadmin(self):
        UserModel = get_user_model()

        if setting_password.value is None:
            password = UserModel.objects.make_random_password()
        else:
            password = setting_password.value

        try:
            UserModel.objects.get(
                **{UserModel.USERNAME_FIELD: setting_username.value}
            )
        except UserModel.DoesNotExist:
            logger.info(
                'Creating superuser -- login: %s, email: %s, password: %s',
                setting_username.value, setting_email.value, password
            )
            management.call_command(
                'createsuperuser',
                **{
                    UserModel.USERNAME_FIELD: setting_username.value,
                    'email': setting_email.value,
                    'interactive': False
                }
            )

            account = UserModel.objects.get(
                **{UserModel.USERNAME_FIELD: setting_username.value}
            )
            account.set_password(raw_password=password)
            account.save()
            # Store the auto admin password properties to display the
            # first login message
            auto_admin_properties, created = self.get_or_create()  # NOQA
            auto_admin_properties.account = account
            auto_admin_properties.password = password
            auto_admin_properties.password_hash = account.password
            auto_admin_properties.save()
        else:
            logger.error(
                'Super admin user already exists. -- login: %s',
                setting_username.value
            )
